/*
   makeqif.c
   
   by Bill Kendrick

   bill@newbreedsoftware.com
   http://www.newbreedsoftware.com/bill/

   kendric@terminator.rs.itd.umich.edu
   ftp://atari.archive.umich.edu/atari/8bit/
   
   October 2, 1997 - May 30, 1999
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#define False 0
#define True 1

#define DESC_FILE ".blurb"
#define QIF_FILE "atari8.qif"
#define MAX_DESC_LEN 1024
#define FAKE_BLURBS True

char * ignore[] = {"incoming"};
int num_ignore = 1;


void traverse(char * path, FILE * fout);


int main(int argc, char * argv[])
{
  FILE * fout;
  
  
  fout = fopen(QIF_FILE, "w");
  if (fout == NULL)
    {
      perror(QIF_FILE);
      exit(1);
    }
  
  printf("The following files had no %s's:\n\n", DESC_FILE);
  
  traverse("", fout);
  
  fclose(fout);
}


void traverse(char * path, FILE * fout)
{
  FILE * fi;
  DIR * dirp;
  struct dirent * entry;
  struct stat buf;
  char filename[1024], descfile[1024], date[10];
  off_t size;
  time_t st_mtime;
  int c, len, ok, i, x, blurb_stat;
  
  
  dirp = opendir(path);
  
  do
    {
      entry = readdir(dirp);
      
      if (entry != NULL)
	{
	  if (strcmp(entry->d_name, ".") != 0 &&
	      strcmp(entry->d_name, "..") != 0 &&
	      entry->d_name[strlen(entry->d_name) - 1] != '~' &&
	      strstr(entry->d_name, DESC_FILE) == NULL)
	    {
	      ok = 1;
	      
	      for (i = 0; i < num_ignore; i++)
		{
		  if (strstr(entry->d_name, ignore[i]) != NULL)
		    ok = 0;
		}
	    }
	  else
	    ok = 0;
	  
	  if (ok == 1)
	    {
	      strcpy(filename, path);
	      strcat(filename, entry->d_name);
	      
	      stat(filename, &buf);
	      
	      size = buf.st_size;
	      
	      st_mtime = buf.st_mtime;
	      strftime(date, 7, "%y%m%d", localtime(&st_mtime));
	      
	      if (S_ISDIR(buf.st_mode) != 0)
		{
		  strcat(filename, "/");
		  traverse(filename, fout);
		}
	      else
		{
		  strcpy(descfile, filename);
		  if (strchr(descfile, '.') != NULL)
		    strcpy(strchr(descfile, '.'), DESC_FILE);
		  else
		    strcat(descfile, DESC_FILE);
		  
		  blurb_stat = stat(descfile, &buf);
	      
		  if (blurb_stat == -1)
		    printf("%s\n", filename);
	      
		  if (blurb_stat != -1 || FAKE_BLURBS == True)
		    {
		      /* Determine newer date.. file or blurb? */
		      
		      if (buf.st_mtime > st_mtime)
			{
			  st_mtime = buf.st_mtime;
			  strftime(date, 7, "%y%m%d", localtime(&st_mtime));
			}
		      
		      fprintf(fout, "%s\t", filename);
		      fprintf(fout, "%s\t", date);
		      fprintf(fout, "%d\t", size);
		      
		      if (blurb_stat != -1)
			{
			  fi = fopen(descfile,"r");
			  if (fi == NULL)
			    {
			      fprintf(stderr,
				      "This shouldn't have happened!\n");
			      perror(descfile);
			      exit(1);
			    }
			  
			  len = 0;
			  
			  x = 0;
			  
			  do
			    {
			      c = fgetc(fi);
			      
			      if (c != EOF)
				{
				  if (c == '\n' || c == 13 || c == ' ')
				    {
				      c = ' ';
				      x = 0;
				    }
				  
				  putc(c, fout);
				  
				  if (x == 60)
				    {
				      x = 0;
				      putc(32, fout);
				    }
				  
				  x++;
				  len++;
				}
			    }
			  while (c != EOF && len < MAX_DESC_LEN);
			  fclose(fi);
			  
			  if (c != EOF)
			    fprintf(fout, "...");
			  
			  putc('\n', fout);
			}
		      else
			{
			  fprintf(fout, "%s\n", filename);
			}
		    }
		}
	    }
	}
    }
  while (entry != NULL);
  
  closedir(dirp);
}
